from ._version import __version__  # noqa: F401
from .extension import load_jupyter_server_extension  # noqa: F401


def _jupyter_server_extension_paths():
    return [{"module": "jupyterlab_autoversion.extension"}]



# for Coverage
import os
import os.path
import tornado.web
from git import Repo
from unittest.mock import MagicMock
from jupyterlab_autoversion.storage.git.handlers import GitGetHandler, GitRestoreHandler
from tempfile import TemporaryDirectory


class TestExtension:
    def test_git_get_handler(self):
        app = tornado.web.Application()
        m = MagicMock()

        with TemporaryDirectory() as d:
            repo_root = os.path.join(d, ".autoversion")
            if not os.path.exists(repo_root):
                os.mkdir(repo_root)

            repo = Repo.init(repo_root)

            def get_argument(name, default):
                if name == "id":
                    return ""
                if name == "path":
                    return ""

            h = GitGetHandler(app, m, repo=repo)
            h._transforms = []
            h.get_argument = get_argument
            h.get()

            def get_argument(name, default):
                if name == "id":
                    return ""
                if name == "path":
                    return "test"

            h = GitGetHandler(app, m, repo=repo)
            h._transforms = []
            h.get_argument = get_argument
            h.get()

            def get_argument(name, default):
                if name == "id":
                    return "test"
                if name == "path":
                    return "test"

            h = GitGetHandler(app, m, repo=repo)
            h._transforms = []
            h.get_argument = get_argument
            h.get()

    def test_git_restore(self):
        app = tornado.web.Application()
        m = MagicMock()

        with TemporaryDirectory() as d:
            repo_root = os.path.join(d, ".autoversion")
            if not os.path.exists(repo_root):
                os.mkdir(repo_root)

            repo = Repo.init(repo_root)

            def get_argument(name, default):
                if name == "id":
                    return ""
                if name == "path":
                    return ""
                else:
                    return 0

            h = GitRestoreHandler(app, m, repo=repo)
            h._transforms = []
            h.get_argument = get_argument
            h.get()

            def get_argument(name, default):
                if name == "id":
                    return ""
                if name == "path":
                    return "test"
                else:
                    return 0

            h = GitRestoreHandler(app, m, repo=repo)
            h._transforms = []
            h.get_argument = get_argument
            h.get()

            def get_argument(name, default):
                if name == "id":
                    return "test"
                if name == "path":
                    return "test"
                else:
                    return 0

            h = GitRestoreHandler(app, m, repo=repo)
            h._transforms = []
            h.get_argument = get_argument
            h.get()



from unittest.mock import MagicMock
from jupyterlab_autoversion.extension import load_jupyter_server_extension


class TestExtension:
    def setup(self):
        pass
        # setup() before each test method

    def teardown(self):
        pass
        # teardown() after each test method

    @classmethod
    def setup_class(cls):
        pass
        # setup_class() before any methods in this class

    @classmethod
    def teardown_class(cls):
        pass
        # teardown_class() after any methods in this class

    def test_load_jupyter_server_extension(self):

        m = MagicMock()

        m.web_app.settings = {}
        m.web_app.settings["base_url"] = "/test"
        m.config = {"JupyterLabAutoversion": {"backend": "git"}}
        load_jupyter_server_extension(m)



import hashlib
import os
import os.path
import nbformat
import tornado.web
from git import Git
from notebook.base.handlers import IPythonHandler


class GitGetHandler(IPythonHandler):
    def initialize(self, repo):
        self.repo = repo

    @tornado.web.authenticated
    def get(self):
        path = self.get_argument("path", "")
        id = self.get_argument("id", "")

        if not id and not path:
            self.finish({"id": "", "last": ""})
            return

        if not id and path:
            sha = hashlib.sha256()
            sha.update(path.encode())
            id = sha.hexdigest()

        last = list(
            reversed(
                [
                    [x.commit.hexsha, x.commit.authored_date * 1000] + x.name.split("-")
                    for x in self.repo.tags
                    if id in x.name
                ]
            )
        )
        self.finish({"id": id, "versions": last})
        return


class GitRestoreHandler(IPythonHandler):
    def initialize(self, repo):
        self.repo = repo

    @tornado.web.authenticated
    def get(self):
        path = self.get_argument("path", "")
        id = self.get_argument("id", "")
        version = int(self.get_argument("version", 0))

        if not id and not path:
            self.finish({"id": "", "version": -1, "contents": {}})
            return

        if not id and path:
            sha = hashlib.sha256()
            sha.update(path.encode())
            id = sha.hexdigest()

        try:
            tag = self.repo.tags["%s-%d" % (id, version)]
        except IndexError:
            self.finish({"id": "", "version": -1, "contents": {}})
            return

        past = self.repo.tags[-1]

        git = Git(self.repo.working_tree_dir)
        git.checkout(tag)

        path = os.path.join(self.repo.working_tree_dir, id)
        nb = os.path.join(path, "NOTEBOOK")

        if os.path.exists(path):
            if os.path.exists(nb):
                nb = nbformat.read(nb, 4)
            else:
                nb = {}
        else:
            nb = {}

        self.finish({"id": id, "version": version, "nb": nb})

        git.checkout(past)



import os
import os.path
from git import Repo
from functools import partial
from notebook.utils import url_path_join

from .handlers import GitGetHandler, GitRestoreHandler
from .hook import post_save_autocommit_git


def initialize(nb_server_app):
    web_app = nb_server_app.web_app
    base_url = web_app.settings["base_url"]

    repo_root = os.path.join(os.path.abspath(os.curdir), ".autoversion")
    if not os.path.exists(repo_root):
        os.mkdir(repo_root)
        repo = Repo.init(repo_root)
    else:
        repo = Repo(repo_root)

    ignore_root = os.path.join(os.path.abspath(os.curdir), ".gitignore")
    if os.path.exists(ignore_root):
        with open(ignore_root, "r+") as fp:
            add = True
            for line in fp:
                if ".autoversion" in line:
                    add = False
            if add:
                fp.write("\n.autoversion\n")
    else:
        with open(ignore_root, "w") as fp:
            fp.write("\n.autoversion\n")

    context = {"repo": repo}
    handlers = [
        (url_path_join(base_url, "autoversion/get"), GitGetHandler, context),
        (url_path_join(base_url, "autoversion/restore"), GitRestoreHandler, context),
    ]

    return partial(post_save_autocommit_git, repo=repo), handlers



from codecs import open
from os import path

from jupyter_packaging import wrap_installers, npm_builder, get_data_files
from setuptools import find_packages, setup

pjoin = path.join
name = "jupyterlab_autoversion"
here = path.abspath(path.dirname(__file__))
jshere = path.abspath(pjoin(path.dirname(__file__), "js"))

with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read().replace("\r\n", "\n")

requires = [
    "GitPython>=2.1.11",
    "jupyterlab>=3.0.0",
]

requires_test = [
    "pytest>=4.3.0",
    "pytest-cov>=2.6.1",
]

requires_dev = (
    requires
    + requires_test
    + [
        "black>=20.8b1",
        "bump2version>=1.0.0",
        "check-manifest",
        "flake8>=3.7.8",
        "flake8-black>=0.2.1",
        "jupyter_packaging",
        "Sphinx>=1.8.4",
        "sphinx-markdown-builder>=0.5.2",
    ]
)

ext_path = pjoin(here, name, "extension")
lab_path = pjoin(here, name, "labextension")

# Representative files that should exist after a successful build
jstargets = [
    pjoin(jshere, "lib", "index.js"),
]

data_spec = [
    (
        "share/jupyter/labextensions/jupyterlab_autoversion",
        lab_path,
        "**",
    ),
    ("etc/jupyter/jupyter_server_config.d", ext_path, "*.json"),
]

ensured_targets = [
    pjoin(here, "jupyterlab_autoversion", "labextension", "package.json"),
]

builder = npm_builder(build_cmd="build:all", path=jshere)

setup(
    name=name,
    version="0.3.3",
    description="Automatically version jupyter notebooks on save",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/timkpaine/jupyterlab_autoversion",
    author="Tim Paine",
    author_email="t.paine154@gmail.com",
    license="Apache 2.0",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Framework :: Jupyter",
        "Framework :: Jupyter :: JupyterLab",
    ],
    platforms="Linux, Mac OS X, Windows",
    keywords=["Jupyter", "Jupyterlab", "Widgets", "IPython", "Graph", "Data", "DAG"],
    cmdclass=wrap_installers(
        post_develop=builder, pre_dist=builder, ensured_targets=ensured_targets
    ),
    data_files=get_data_files(data_spec),
    packages=find_packages(
        exclude=[
            "tests",
        ]
    ),
    install_requires=requires,
    test_suite="jupyterlab_autoversion.tests",
    tests_require=requires_test,
    extras_require={
        "dev": requires_dev,
        "develop": requires_dev,
    },
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.7",
)



import hashlib
import json
import os
import os.path


def post_save_autocommit_git(repo, model, *args, **kwargs):
    """convert notebooks to Python script after save with nbconvert
    replaces `jupyter notebook --script`
    """
    if model["type"] != "notebook":
        return

    if model.get("content") and model.get("path"):
        id = model["content"]["metadata"].get("autoversion", "")

        if not id:
            sha = hashlib.sha256()
            sha.update(model["path"].encode())
            id = sha.hexdigest()

        path = os.path.join(repo.working_tree_dir, id)

        if not os.path.exists(path):
            os.mkdir(path)

        nb = os.path.join(path, "NOTEBOOK")
        with open(nb, "w") as fp:
            fp.write(json.dumps(model["content"]))

        last = len([x for x in repo.tags if id in x.name])

        repo.index.add([nb])
        repo.index.commit("%s-%d" % (id, last))
        repo.create_tag("%s-%d" % (id, last))



from notebook.utils import url_path_join


def load_jupyter_server_extension(nb_server_app):
    """
    Called when the extension is loaded.

    Args:
        nb_server_app (NotebookWebApplication): handle to the Notebook webserver instance.
    """
    web_app = nb_server_app.web_app
    base_url = web_app.settings["base_url"]
    host_pattern = ".*$"

    backend = nb_server_app.config.get("JupyterLabAutoversion", {}).get(
        "backend", "git"
    )

    if backend not in ("git", "s3", "sql"):
        raise Exception(
            "jupyterlab_autoversion backend not recognized: {}".format(backend)
        )

    if backend == "s3":
        from .storage.s3 import initialize
    elif backend == "sql":
        from .storage.sql import initialize
    else:
        from .storage.git import initialize

    hook, handlers = initialize(nb_server_app)

    print(
        "Installing jupyterlab_autoversion handler on path %s"
        % url_path_join(base_url, "autoversion")
    )

    web_app.add_handlers(host_pattern, handlers)

    if callable(nb_server_app.contents_manager.pre_save_hook):
        # grab existing hook, and add on top
        temp_hook = nb_server_app.contents_manager.pre_save_hook

        def _hook(*args, **kwargs):
            temp_hook(*args, **kwargs)
            hook(*args, **kwargs)

        nb_server_app.contents_manager.pre_save_hook = _hook
    else:
        nb_server_app.contents_manager.pre_save_hook = hook
